import cgi
import wsgiref.handlers

from google.appengine.api import users
from google.appengine.ext import webapp
from google.appengine.ext import db
from google.appengine.ext.webapp import template
from google.appengine.api import urlfetch
from google.appengine.api.datastore_errors import BadKeyError

import urllib

import logging

import simplejson

from models import ChipUser
from common import *

class Bookmark:
  def __init__(self, title, link, description, guid):
    self.title = title
    self.link = link
    self.description = description
    self.guid = guid

class RssFeed(webapp.RequestHandler):
  def get(self,key=None,labels=None):
    set_content_type(self.response, "text/xml")
    logging.info("inside rss handler")
    
    try:
      chipuser = db.get(key)
      
      form_data = urllib.urlencode({"convertjson":"true",
                                    "agent":"ext",
                                    "clientName":chipuser.username,
                                    "clientPass":chipuser.password})
      result = urlfetch.fetch(url="https://www.chipmark.com/Login",
                              payload=form_data,
                              method=urlfetch.POST,
                              headers={'Content-Type': 'application/x-www-form-urlencoded'})
      
      #check and make sure we had a successful login
      data = simplejson.loads(result.content)
      if data['xml']['LoggedInAs']:
        cookie = result.headers['set-cookie']
        
        result = urlfetch.fetch(url="https://www.chipmark.com/GetFoldersAndLinks?agent=ext&convertjson=true",
                                payload=form_data,
                                method=urlfetch.GET,
                                headers={'Cookie': cookie})
        data = simplejson.loads(result.content)
        if data['xml']['Result'] == 'SUCCESS':
          logging.info("get folders and links loaded")
          # only return the public bookmarks that a user owns
          #we have to look at the parent folder to see if it is owned by this user
          folder_id_to_owner_name = {}
          for f in data['xml']['FolderList']['Folder']:
            folder_id_to_owner_name[f['id']] = f['ownerName']
          logging.info("folders parsed")
          bookmarks = []
          
          if labels:
            labels = labels.split("%2C")
            if len(labels) > 0 and labels[-1] == '':
              labels.pop()
          
          for b in data['xml']['BookmarkList'].get('Bookmark',[]):
            #bug in the server that some how allows for bookmarks not to have a valid parent
            #these should be skipped over
            parent_folder = folder_id_to_owner_name.get(b['linkFolderParentID'],None)            
            if b["permission"] == "public" and parent_folder == chipuser.username:
              #only include bookmarks that have the labels in 'labels'. 
              #they may have more labels bust at must at least
              #contain all those passwed in on the end
              if labels is not None:
                label_list = b["LabelList"].get("Label",[])
                #make sure we have a list
                if len(label_list) == 1:
                  label_list = [label_list]
                logging.info(label_list)
                label_list = [l['name'] for l in label_list]
                intersection = set(labels).intersection(label_list)
                if len(intersection) != len(labels):
                  continue
              bookmarks.append( (int(b['id']),b['Title'],b['Link'],b['Description']) )
              
          logging.info("bookmarks parsed")
          #make sure the most recently added is at the top
          bookmarks.sort()
          bookmarks.reverse()
          bookmarks = [Bookmark(bookmark[1],bookmark[2],bookmark[3],str(bookmark[0])+str(bookmark[2])) for bookmark in bookmarks]
          render_to_response(self.response, 'templates/rss.xml', {"title":chipuser.username,"items":bookmarks})
        else:
          render_to_response(self.response, 'templates/error_rss.xml', {"msg":"failed to retrieve bookmarks"})
      else:
        render_to_response(self.response, 'templates/error_rss.xml', {"msg":"incorrect chipmark login"})

    except BadKeyError:
      logging.error("key did not exist " + key)
      render_to_response(self.response,'templates/error_rss.xml', {"msg":"key does not exist"})

def main():
  application = webapp.WSGIApplication(
                                       [(r'/rss/(.+)/(.*)', RssFeed),
                                        (r'/rss/(.+)', RssFeed),
										],debug=True)
  wsgiref.handlers.CGIHandler().run(application)

if __name__ == "__main__":
  main()
